package com.example.emr;

import java.util.Arrays;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.amazonaws.auth.BasicAWSCredentials;
import com.amazonaws.services.elasticmapreduce.AmazonElasticMapReduceClient;
import com.amazonaws.services.elasticmapreduce.model.DescribeJobFlowsRequest;
import com.amazonaws.services.elasticmapreduce.model.DescribeJobFlowsResult;
import com.amazonaws.services.elasticmapreduce.model.HadoopJarStepConfig;
import com.amazonaws.services.elasticmapreduce.model.JobFlowDetail;
import com.amazonaws.services.elasticmapreduce.model.JobFlowInstancesConfig;
import com.amazonaws.services.elasticmapreduce.model.PlacementType;
import com.amazonaws.services.elasticmapreduce.model.RunJobFlowRequest;
import com.amazonaws.services.elasticmapreduce.model.RunJobFlowResult;
import com.amazonaws.services.elasticmapreduce.model.StepConfig;
import com.amazonaws.services.elasticmapreduce.model.StepDetail;
import com.amazonaws.services.elasticmapreduce.model.TerminateJobFlowsRequest;

public class EMRWordCount {
	private static final Log LOG = LogFactory.getLog(EMRWordCount.class);

	private static final String ACCESS_KEY = "XXXXX";
	private static final String SECREC_KEY = "XXXXXXXXXX";

	private static final int SLEEP_PERIOD = 1000 * 10;

	private static final String MAP_REDUCE_NAME = "EMR Job";
	private static final String ACTION_ON_FAILURE = "TERMINATE_JOB_FLOW";
	private static final String STEP_DETAIL_STATUS_PENDING = "PENDING";
	private static final String STEP_DETAIL_STATUS_RUNNING = "RUNNING";
	private static final String STEP_DETAIL_STATUS_COMPLETED = "COMPLETED";

	private static final String INSTANCE = "m1.small";
	private static final String AMI_VERSION = "2.0";
	private static final String HADOOP_VERSION = "0.20.205";
	private static final String ENDPOINT = "ap-northeast-1.elasticmapreduce.amazonaws.com";
	private static final String AVAILABILITY_ZONE = "ap-northeast-1a";
	private static final String KEY_PAIR_NAME = "nt-rd";
	private static final int INSTANCE_NUMBER = 4;
	private static final String MAPREDUCE_JAR = "s3://nt-rd-sample/jar/hadoop-examples-0.20.205.0.jar";
	private static final String EMR_LOG_URI = "s3://nt-rd-sample/logs/";


	private AmazonElasticMapReduceClient emr;
	private String jobFlowId;

	public EMRWordCount() {
		emr = new AmazonElasticMapReduceClient(new BasicAWSCredentials(ACCESS_KEY, SECREC_KEY));
		emr.setEndpoint(ENDPOINT);
	}

	public static void main(String[] args) {
		LOG.info("begin main");
		
		EMRWordCount wordcount = new EMRWordCount();
		wordcount.execute(args);

		LOG.info("end main");
	}

	private void execute(String[] args) {
		String s3InputPath = args[0];
		String s3OutputPath = args[1];
		
		String jobName = "wordcount";
		String stepName = "job";

		LOG.info("creating step...");
		HadoopJarStepConfig hadoopJarStepConfig =
				new HadoopJarStepConfig()
					.withJar(MAPREDUCE_JAR)
					.withArgs(Arrays.asList(jobName, s3InputPath, s3OutputPath));
		
		StepConfig stepConfig =
				new StepConfig()
					.withName(stepName)
					.withActionOnFailure(ACTION_ON_FAILURE)
					.withHadoopJarStep(hadoopJarStepConfig);

		LOG.info("creating Job Flow...");
		JobFlowInstancesConfig jobFlowInstancesConfig =
				new JobFlowInstancesConfig()
					.withEc2KeyName(KEY_PAIR_NAME)
					.withHadoopVersion(HADOOP_VERSION)
					.withPlacement(new PlacementType(AVAILABILITY_ZONE))
					.withInstanceCount(INSTANCE_NUMBER)
					.withKeepJobFlowAliveWhenNoSteps(true)
					.withMasterInstanceType(INSTANCE)
					.withSlaveInstanceType(INSTANCE);
		
		RunJobFlowRequest runJobFlowRequest =
				new RunJobFlowRequest()
					.withAmiVersion(AMI_VERSION)
					.withName(MAP_REDUCE_NAME)
					.withSteps(stepConfig)
					.withLogUri(EMR_LOG_URI)
					.withInstances(jobFlowInstancesConfig);

		LOG.info("running Job Flow...");
		RunJobFlowResult result = emr.runJobFlow(runJobFlowRequest);
		jobFlowId = result.getJobFlowId();

		String resultStatus = waitingStepEnded(stepName);
		
		if (STEP_DETAIL_STATUS_COMPLETED.equals(resultStatus)) {
			LOG.info("completed");
		} else {
			LOG.error("error");
		}

		if (jobFlowId != null) {
			LOG.info("terminating Job Flow...");
			TerminateJobFlowsRequest request = new TerminateJobFlowsRequest().withJobFlowIds(jobFlowId);
			emr.terminateJobFlows(request);
			jobFlowId = null;
		}
	}
	
	private String waitingStepEnded(String stepName) {
		LOG.info("waiting for step ended...");
		String stepStatus = getStatus(stepName);
		LOG.info("state: " + stepStatus);
		while (STEP_DETAIL_STATUS_PENDING.equals(stepStatus)
				|| STEP_DETAIL_STATUS_RUNNING.equals(stepStatus)) {
			try {
				Thread.sleep(SLEEP_PERIOD);
			} catch (Exception e) {
			}

			stepStatus = getStatus(stepName);
			LOG.info("state: " + stepStatus);
		}
		return stepStatus;
	}
	
	private String getStatus(String stepName) {
		LOG.info("getting step status: " + stepName);
		DescribeJobFlowsRequest describeJobFlowsRequest =
				new DescribeJobFlowsRequest().withJobFlowIds(jobFlowId);
		DescribeJobFlowsResult describeJobFlowsResult =
				emr.describeJobFlows(describeJobFlowsRequest);

		for (JobFlowDetail jobFlowDetail : describeJobFlowsResult.getJobFlows()) {
			for (StepDetail stepDetail : jobFlowDetail.getSteps()) {
				if (stepDetail.getStepConfig().getName().equals(stepName)) {
					LOG.info("step found: " + stepName);
					return stepDetail.getExecutionStatusDetail().getState();
				}
			}
		}
		LOG.warn("step not found: " + stepName);
		return null;
	}
}
